@@ -5,13 +5,14 @@ import datetime
 import json
 import threading
 import time
+import traceback
 import typing
 
 import numpy as np
 
 from cracknuts import logger
-from cracknuts.cracker.stateful_cracker import StatefulCracker
-from cracknuts.solver.trace import ScarrTraceDataset, NumpyTraceDataset
+from cracknuts.cracker.cracker_basic import CrackerBasic
+from cracknuts.trace.trace import ScarrTraceDataset, NumpyTraceDataset
 
 
 class AcqProgress:
@@ -21,6 +22,10 @@ class AcqProgress:
 
 
 class Acquisition(abc.ABC):
+    """
+    Trace acquisition process control class
+    """
+
     STATUS_STOPPED: int = 0
     STATUS_TESTING: int = 1
     STATUS_RUNNING: int = 2
@@ -31,7 +36,7 @@ class Acquisition(abc.ABC):
 
     def __init__(
         self,
-        cracker: StatefulCracker,
+        cracker: CrackerBasic,
         trace_count: int = 1000,
         sample_length: int = -1,
         sample_offset: int = 0,
@@ -44,8 +49,32 @@ class Acquisition(abc.ABC):
         file_path: str = "auto",
     ):
         """
+        :param cracker: The controlled Cracker object.
+        :type cracker: CrackerBasic
+        :param trace_count: The number of traces to acquire.
+        :type trace_count: int
         :param sample_length: the sample length to be acquired,
-                              it will set to `cracker.current_count().sample_len` if it is -1
+                              If it is -1, it will be set to the value of `cracker.current_count().sample_len`.
+        :type sample_length: int
+        :param sample_offset: the sample offset to be acquired.
+        :type sample_offset: int
+        :param data_length: the data length to be acquired.
+        :type data_length: int
+        :param trigger_judge_timeout: The trigger judge timeout in seconds.
+        :type trigger_judge_timeout: float
+        :param trigger_judge_wait_time: The trigger judge wait time in seconds.
+        :type trigger_judge_wait_time: float
+        :param do_error_handler_strategy: The strategy to handle error handling.
+                                          0: Exit immediately, 1: Exit after exceeding the error count.
+        :type do_error_handler_strategy: int
+        :param do_error_max_count: The maximum number of error count.
+                                   If `do_error_handler_strategy` is 0, this setting is invalid.
+        :type do_error_max_count: int
+        :param file_format: The file format of the trace dataset. "scarr" or "numpy".
+        :type file_format: str
+        :param file_path: The file path of the trace dataset. If set to "auto", a folder with a timestamp format
+                          will be created in the current working directory to save the data.
+        :type file_path: str
         """
         self._logger = logger.get_logger(self)
         self._last_wave: dict[int, np.ndarray] | None = {1: np.zeros(1)}
@@ -53,7 +82,7 @@ class Acquisition(abc.ABC):
         self._current_trace_count = 1
         self._run_thread_pause_event: threading.Event = threading.Event()
 
-        self.cracker: StatefulCracker = cracker
+        self.cracker: CrackerBasic = cracker
         self.trace_count: int = trace_count
         self.sample_length = sample_length
         self.sample_offset: int = sample_offset
@@ -72,7 +101,7 @@ class Acquisition(abc.ABC):
     def get_status(self):
         return self._status
 
-    def set_cracker(self, cracker: StatefulCracker):
+    def set_cracker(self, cracker: CrackerBasic):
         self.cracker = cracker
 
     def on_status_changed(self, callback: typing.Callable[[int], None]) -> None:
@@ -103,6 +132,34 @@ class Acquisition(abc.ABC):
         file_format: str | None = "scarr",
         file_path: str | None = "auto",
     ):
+        """
+        Start run mode in the background.
+        The parameters configured here will override the settings specified in `__init__`.
+
+        :param count: The number of traces to acquire.
+        :type count: int
+        :param sample_length: The sample length to be acquired.
+        :type sample_length: int
+        :param sample_offset: The sample offset to be acquired.
+        :type sample_offset: int
+        :param data_length: The data length to be acquired.
+        :type data_length: int
+        :param trigger_judge_wait_time: The trigger judge wait time in seconds.
+        :type trigger_judge_wait_time: float
+        :param trigger_judge_timeout: The trigger judge timeout in seconds.
+        :type trigger_judge_timeout: float
+        :param do_error_max_count: The maximum number of error count.
+                                   If `do_error_handler_strategy` is 0, this setting is invalid.
+        :type do_error_max_count: int
+        :param do_error_handler_strategy: The strategy to handle error handling.
+                                          0: Exit immediately, 1: Exit after exceeding the error count.
+        :type do_error_handler_strategy: int
+        :param file_format: The file format of the trace dataset. "scarr" or "numpy".
+        :type file_format: str
+        :param file_path: The file path of the trace dataset. If set to "auto", a folder with a timestamp format
+                          will be created in the current working directory to save the data.
+        :type file_path: str
+        """
         if self._status < 0:
             self.resume()
         else:
@@ -130,7 +187,33 @@ class Acquisition(abc.ABC):
         do_error_max_count: int | None = None,
         do_error_handler_strategy: int | None = None,
         file_format: str | None = "scarr",
+        file_path: str | None = "auto",
     ):
+        """
+        Start run mode in the foreground, which will cause blocking.
+
+        :param count: The number of traces to acquire.
+        :type count: int
+        :param sample_length: The sample length to be acquired.
+        :type sample_length: int
+        :param sample_offset: The sample offset to be acquired.
+        :type sample_offset: int
+        :param trigger_judge_wait_time: The trigger judge wait time in seconds.
+        :type trigger_judge_wait_time: float
+        :param trigger_judge_timeout: The trigger judge timeout in seconds.
+        :type trigger_judge_timeout: float
+        :param do_error_max_count: The maximum number of error count.
+                                   If `do_error_handler_strategy` is 0, this setting is invalid.
+        :type do_error_max_count: int
+        :param do_error_handler_strategy: The strategy to handle error handling.
+                                          0: Exit immediately, 1: Exit after exceeding the error count.
+        :type do_error_handler_strategy: int
+        :param file_format: The file format of the trace dataset. "scarr" or "numpy".
+        :type file_format: str
+        :param file_path: The file path of the trace dataset. If set to "auto", a folder with a timestamp format
+                          will be created in the current working directory to save the data.
+        :type file_path: str
+        """
         try:
             self._do_run(
                 test=False,
@@ -142,6 +225,7 @@ class Acquisition(abc.ABC):
                 do_error_max_count=do_error_max_count,
                 do_error_handler_strategy=do_error_handler_strategy,
                 file_format=file_format,
+                file_path=file_path,
             )
             self._logger.debug("stop by timeout.")
         except KeyboardInterrupt:
@@ -158,6 +242,25 @@ class Acquisition(abc.ABC):
         do_error_max_count: int | None = None,
         do_error_handler_strategy: int | None = None,
     ):
+        """
+        Start test mode in background.
+
+        :param count: The number of traces to acquire. normally this value should be -1.
+        :type count: int
+        :param sample_length: The sample length to be acquired.
+        :type sample_length: int
+        :param sample_offset: The sample offset to be acquired.
+        :type sample_offset: int
+        :param trigger_judge_wait_time: The trigger judge wait time in seconds.
+        :type trigger_judge_wait_time: float
+        :param trigger_judge_timeout: The trigger judge timeout in seconds.
+        :type trigger_judge_timeout: float
+        :param do_error_max_count: The maximum number of error count.
+                                   If `do_error_handler_strategy` is 0, this setting is invalid.
+        :type do_error_max_count: int
+        :param do_error_handler_strategy: The strategy to handle error handling.
+                                          0: Exit immediately, 1: Exit after exceeding the error count.
+        """
         if self._status < 0:
             self._logger.debug("Resume to test mode.")
             self.resume()
@@ -184,6 +287,25 @@ class Acquisition(abc.ABC):
         do_error_max_count: int | None = None,
         do_error_handler_strategy: int | None = None,
     ):
+        """
+        Start test mode in foreground, which will cause blocking.
+
+        :param count: The number of traces to acquire. normally this value should be -1.
+        :type count: int
+        :param sample_length: The sample length to be acquired.
+        :type sample_length: int
+        :param sample_offset: The sample offset to be acquired.
+        :type sample_offset: int
+        :param trigger_judge_wait_time: The trigger judge wait time in seconds.
+        :type trigger_judge_wait_time: float
+        :param trigger_judge_timeout: The trigger judge timeout in seconds.
+        :type trigger_judge_timeout: float
+        :param do_error_max_count: The maximum number of error count.
+                                   If `do_error_handler_strategy` is 0, this setting is invalid.
+        :type do_error_max_count: int
+        :param do_error_handler_strategy: The strategy to handle error handling.
+                                          0: Exit immediately, 1: Exit after exceeding the error count.
+        """
         try:
             self._do_run(
                 test=True,
@@ -200,16 +322,28 @@ class Acquisition(abc.ABC):
             self.stop()
 
     def pause(self):
+        """
+        Pause test or run mode.
+
+        """
         if self._status > 0:
             self._run_thread_pause_event.clear()
             self._status = self._status * self._STATUS_PAUSED_SWITCH
 
     def resume(self):
+        """
+        Resume from paused test or run mode.
+
+        """
         if self._status < 0:
             self._run_thread_pause_event.set()
             self._status = self._status * self._STATUS_PAUSED_SWITCH
 
     def stop(self):
+        """
+        Stop test or run mode.
+
+        """
         if not self._run_thread_pause_event.is_set():
             self._run_thread_pause_event.set()
         self._status = self.STATUS_STOPPED
@@ -261,7 +395,10 @@ class Acquisition(abc.ABC):
         file_path: str | None = "auto",
     ):
         if self._status > 0:
-            raise Exception(f'AcquisitionTemplate is already running in {'run' if self._status == 2 else 'test'} mode.')
+            self._logger.warning(
+                f'AcquisitionTemplate is already running in {'run' if self._status == 2 else 'test'} mode.'
+            )
+            return
 
         if not test:
             self._status = self.STATUS_RUNNING
@@ -308,7 +445,8 @@ class Acquisition(abc.ABC):
 
         cracker_version = self.cracker.get_version()
         if persistent:
-            channel_count = len(self.cracker.get_current_config().osc_analog_channel_enable.keys())
+            channel_names = [str(k) for k in self.cracker.get_current_config().osc_analog_channel_enable.keys()]
+            print(channel_names)
             if self.sample_length == -1:
                 sample_length = self.cracker.get_current_config().osc_sample_len
             else:
@@ -324,7 +462,7 @@ class Acquisition(abc.ABC):
             if file_format == "scarr":
                 dataset = ScarrTraceDataset.new(
                     file_path,
-                    channel_count,
+                    channel_names,
                     self.trace_count,
                     sample_length,
                     self.data_length,
@@ -333,7 +471,7 @@ class Acquisition(abc.ABC):
             elif file_format == "numpy":
                 dataset = NumpyTraceDataset.new(
                     file_path,
-                    channel_count,
+                    channel_names,
                     self.trace_count,
                     sample_length,
                     self.data_length,
@@ -356,7 +494,7 @@ class Acquisition(abc.ABC):
             try:
                 data = self.do()
             except Exception as e:
-                self._logger.error("Do error: %s", e.args)
+                self._logger.error(f"Do error <{e}>:\n {traceback.format_exc()}")
                 do_error_count += 1
                 if self.do_error_handler_strategy == self.DO_ERROR_HANDLER_STRATEGY_EXIT:
                     self._logger.error("Exit with do error")
@@ -399,13 +537,16 @@ class Acquisition(abc.ABC):
                 time.sleep(self.trigger_judge_wait_time)
             if dataset is not None and self._last_wave is not None:
                 if 1 in self._last_wave.keys():
-                    dataset.set_trace(0, trace_index, self._last_wave[1], np.frombuffer(data, dtype=np.uint8))
+                    dataset.set_trace("1", trace_index, self._last_wave[1], np.frombuffer(data, dtype=np.uint8))
                 elif 2 in self._last_wave.keys():
-                    dataset.set_trace(1, trace_index, self._last_wave[2], np.frombuffer(data, dtype=np.uint8))
+                    dataset.set_trace("2", trace_index, self._last_wave[2], np.frombuffer(data, dtype=np.uint8))
             self._post_do()
             trace_index += 1
             self._current_trace_count = trace_index
             self._progress_changed(AcqProgress(trace_index, self.trace_count))
+            # Reduce the execution frequency in test mode.
+            if not persistent:
+                time.sleep(2)
 
         if dataset is not None:
             dataset.dump()
@@ -469,11 +610,15 @@ class Acquisition(abc.ABC):
 
     def config_cracker(self): ...
 
-    def pre_init(self):
-        self.cracker.sync_config_to_cracker()
+    def pre_init(self): ...
 
     @abc.abstractmethod
-    def init(self): ...
+    def init(self):
+        """
+        The `init` logic, which the user should implement in the subclass.
+
+        """
+        ...
 
     def _post_init(self): ...
 
@@ -483,7 +628,12 @@ class Acquisition(abc.ABC):
         self.cracker.osc_single()
 
     @abc.abstractmethod
-    def do(self): ...
+    def do(self):
+        """
+        The `do` logic, which the user should implement in the subclass.
+
+        """
+        ...
 
     def _do(self):
         try:
@@ -496,9 +646,8 @@ class Acquisition(abc.ABC):
     def _post_do(self): ...
 
     def _is_triggered(self):
-        status, res = self.cracker.osc_is_triggered()
-        res_code = int.from_bytes(res, "big")
-        return res_code == 4
+        _, triggered = self.cracker.osc_is_triggered()
+        return triggered
 
     def _get_waves(self, offset: int, sample_length: int) -> dict[int, np.ndarray]:
         if sample_length == -1:
@@ -522,11 +671,16 @@ class Acquisition(abc.ABC):
         pass
 
     def get_last_wave(self) -> dict[int, np.ndarray] | None:
+        """
+        Get the last wave.
+
+        """
         return self._last_wave
 
     def dump_config(self, path: str = None) -> str | None:
         """
         Dump the current config to a JSON file if a path is specified, or to a JSON string if no path is specified.
+
         :param path: the path to the JSON file
         :return: the content of JSON string or None if no path is specified.
         """
@@ -547,7 +701,8 @@ class Acquisition(abc.ABC):
 
     def load_config_from_file(self, path: str) -> None:
         """
-        Load config from a JSON file
+        Load config from a JSON file.
+
         :param path: the path to the JSON file
         """
         with open(path) as f:
@@ -555,7 +710,8 @@ class Acquisition(abc.ABC):
 
     def load_config_from_str(self, json_str: str) -> None:
         """
-        Load config from a JSON string
+        Load config from a JSON string.
+
         :param json_str: the JSON string
         """
         config = json.loads(json_str)
@@ -567,32 +723,68 @@ class Acquisition(abc.ABC):
         self.do_error_max_count = config["do_error_max_count"]
         self.file_path = config["trace_file_path"]
 
+    def is_running(self):
+        return self._status != self.STATUS_STOPPED
+
     @staticmethod
     def builder():
         return AcquisitionBuilder()
 
 
 class AcquisitionBuilder:
+    """
+    The helper class for create a Sub acquisition class.
+
+    """
+
     def __init__(self):
         self._cracker = None
         self._do_function = lambda _: ...
         self._init_function = lambda _: ...
 
-    def cracker(self, cracker: StatefulCracker):
+    def cracker(self, cracker: CrackerBasic):
+        """
+        Specify the cracker for acquisition.
+
+        :param cracker: the cracker
+        :type cracker: CrackerBasic
+        :return: 'AcquisitionBuilder'
+        """
         self._cracker = cracker
         return self
 
-    def init(self, init_function: typing.Callable[[StatefulCracker], None]):
+    def init(self, init_function: typing.Callable[[CrackerBasic], None]):
+        """
+        The init function of acquisition.
+
+        :param init_function: the init function
+        :type init_function: typing.Callable[[CrackerBasic], None]
+        :return: 'AcquisitionBuilder'
+        """
         if init_function is not None:
             self._init_function = init_function
         return self
 
-    def do(self, do_function: typing.Callable[[StatefulCracker], None]):
+    def do(self, do_function: typing.Callable[[CrackerBasic], None]):
+        """
+        The do function of acquisition.
+
+        :param do_function: the do function
+        :type do_function: typing.Callable[[CrackerBasic], None]
+        :return: 'AcquisitionBuilder'
+        """
         if do_function is not None:
             self._do_function = do_function
         return self
 
     def build(self, **kwargs):
+        """
+        Build the acquisition object.
+
+        :param kwargs: other keyword arguments of Acquisition.
+        :type kwargs: dict
+        :return Acquisition
+        """
         if self._cracker is None:
             raise ValueError("No cracker")
 
